/* puzzle-generator.c generated by valac 0.42.0, the Vala compiler
 * generated from puzzle-generator.vala, do not modify */

/*
 * Copyright (C) 2010-2013 Robert Ancell
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 2 of the License, or (at your option) any later
 * version. See http://www.gnu.org/copyleft/gpl.html the full text of the
 * license.
 */
/* Puzzle generation logic:*/
/* We want to measure a puzzle's difficulty by the number of button presses*/
/* needed to solve it, and have that increase in a controlled manner.*/
/**/
/* Lights Off can be seen as a linear algebra problem over the field GF(2).*/
/* (See e.g. the first page of "Turning Lights Out with Linear Algebra".)*/
/* The linear map from button-press strategies to light configurations*/
/* will in general have a nullspace (of dimension n).*/
/* Thus, any solvable puzzle has 2^n solutions, given by a fixed solution*/
/* plus an element of the nullspace.*/
/* A solution is thus optimal if it presses at most half the buttons*/
/* which make up any element of the nullspace.*/
/**/
/* A basis for the nullspace splits the board into (up to) 2^n regions,*/
/* defined by which basic nullspace elements include a given button.*/
/* (This determines which nullspace elements include the same button.)*/
/* Thus, a solution is optimal if it includes at most half the buttons in any*/
/* region (except the region lying outside all null-sets).*/
/* The converse is not true, but (at least if the regions are even-sized)*/
/* does hold for a large number of puzzles up to the highest difficulty level.*/
/* (Certainly, on average, at most half the lights which belong to at least*/
/* one null set may be used.)*/


#include <glib.h>
#include <glib-object.h>
#include <math.h>
#include <float.h>


#define TYPE_PUZZLE_GENERATOR (puzzle_generator_get_type ())
#define PUZZLE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PUZZLE_GENERATOR, PuzzleGenerator))
#define PUZZLE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PUZZLE_GENERATOR, PuzzleGeneratorClass))
#define IS_PUZZLE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PUZZLE_GENERATOR))
#define IS_PUZZLE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PUZZLE_GENERATOR))
#define PUZZLE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PUZZLE_GENERATOR, PuzzleGeneratorClass))

typedef struct _PuzzleGenerator PuzzleGenerator;
typedef struct _PuzzleGeneratorClass PuzzleGeneratorClass;
typedef struct _PuzzleGeneratorPrivate PuzzleGeneratorPrivate;
enum  {
	PUZZLE_GENERATOR_0_PROPERTY,
	PUZZLE_GENERATOR_NUM_PROPERTIES
};
static GParamSpec* puzzle_generator_properties[PUZZLE_GENERATOR_NUM_PROPERTIES];

struct _PuzzleGenerator {
	GObject parent_instance;
	PuzzleGeneratorPrivate * priv;
};

struct _PuzzleGeneratorClass {
	GObjectClass parent_class;
};

struct _PuzzleGeneratorPrivate {
	gint size;
	gint max_solution_length;
	gint* region_of;
	gint region_of_length1;
	gint _region_of_size_;
	gint* region_size;
	gint region_size_length1;
	gint _region_size_size_;
};


static gint PuzzleGenerator_private_offset;
static gpointer puzzle_generator_parent_class = NULL;

GType puzzle_generator_get_type (void) G_GNUC_CONST;
PuzzleGenerator* puzzle_generator_new (gint size);
PuzzleGenerator* puzzle_generator_construct (GType object_type,
                                             gint size);
gboolean* puzzle_generator_minimal_solution (PuzzleGenerator* self,
                                             gint solution_length,
                                             int* result_length1,
                                             int* result_length2);
static void puzzle_generator_finalize (GObject * obj);


static inline gpointer
puzzle_generator_get_instance_private (PuzzleGenerator* self)
{
	return G_STRUCT_MEMBER_P (self, PuzzleGenerator_private_offset);
}


PuzzleGenerator*
puzzle_generator_construct (GType object_type,
                            gint size)
{
	PuzzleGenerator * self = NULL;
	gint* adj_matrix = NULL;
	gint* _tmp0_;
	gint adj_matrix_length1;
	gint adj_matrix_length2;
	GList* non_pivot_cols = NULL;
	gint ipiv = 0;
	gint* basis_for_ns = NULL;
	GList* _tmp83_;
	gint* _tmp84_;
	gint basis_for_ns_length1;
	gint basis_for_ns_length2;
	gint n = 0;
	GList* _tmp85_;
	GList* _tmp125_;
	gint* _tmp126_;
	gint* _tmp134_;
	gint* _tmp158_;
	gint _tmp158__length1;
	gint _tmp159_;
	self = (PuzzleGenerator*) g_object_new (object_type, NULL);
	self->priv->size = size;
	_tmp0_ = g_new0 (gint, (size * size) * (size * size));
	adj_matrix = _tmp0_;
	adj_matrix_length1 = size * size;
	adj_matrix_length2 = size * size;
	{
		gint x0 = 0;
		x0 = 0;
		{
			gboolean _tmp1_ = FALSE;
			_tmp1_ = TRUE;
			while (TRUE) {
				gint _tmp3_;
				if (!_tmp1_) {
					gint _tmp2_;
					_tmp2_ = x0;
					x0 = _tmp2_ + 1;
				}
				_tmp1_ = FALSE;
				_tmp3_ = x0;
				if (!(_tmp3_ < size)) {
					break;
				}
				{
					gint y0 = 0;
					y0 = 0;
					{
						gboolean _tmp4_ = FALSE;
						_tmp4_ = TRUE;
						while (TRUE) {
							gint _tmp6_;
							if (!_tmp4_) {
								gint _tmp5_;
								_tmp5_ = y0;
								y0 = _tmp5_ + 1;
							}
							_tmp4_ = FALSE;
							_tmp6_ = y0;
							if (!(_tmp6_ < size)) {
								break;
							}
							{
								gint x1 = 0;
								x1 = 0;
								{
									gboolean _tmp7_ = FALSE;
									_tmp7_ = TRUE;
									while (TRUE) {
										gint _tmp9_;
										if (!_tmp7_) {
											gint _tmp8_;
											_tmp8_ = x1;
											x1 = _tmp8_ + 1;
										}
										_tmp7_ = FALSE;
										_tmp9_ = x1;
										if (!(_tmp9_ < size)) {
											break;
										}
										{
											gint y1 = 0;
											y1 = 0;
											{
												gboolean _tmp10_ = FALSE;
												_tmp10_ = TRUE;
												while (TRUE) {
													gint _tmp12_;
													gint dx = 0;
													gint _tmp13_;
													gint _tmp14_;
													gint dy = 0;
													gint _tmp15_;
													gint _tmp16_;
													gint _tmp17_ = 0;
													gint _tmp18_;
													gint _tmp19_;
													gint _tmp20_;
													gint _tmp21_;
													gint* _tmp22_;
													gint _tmp22__length1;
													gint _tmp22__length2;
													gint _tmp23_;
													gint _tmp24_;
													gint _tmp25_;
													gint _tmp26_;
													gint _tmp27_;
													if (!_tmp10_) {
														gint _tmp11_;
														_tmp11_ = y1;
														y1 = _tmp11_ + 1;
													}
													_tmp10_ = FALSE;
													_tmp12_ = y1;
													if (!(_tmp12_ < size)) {
														break;
													}
													_tmp13_ = x0;
													_tmp14_ = x1;
													dx = _tmp13_ - _tmp14_;
													_tmp15_ = y0;
													_tmp16_ = y1;
													dy = _tmp15_ - _tmp16_;
													_tmp18_ = dx;
													_tmp19_ = dx;
													_tmp20_ = dy;
													_tmp21_ = dy;
													if (((_tmp18_ * _tmp19_) + (_tmp20_ * _tmp21_)) <= 1) {
														_tmp17_ = 1;
													} else {
														_tmp17_ = 0;
													}
													_tmp22_ = adj_matrix;
													_tmp22__length1 = adj_matrix_length1;
													_tmp22__length2 = adj_matrix_length2;
													_tmp23_ = x0;
													_tmp24_ = y0;
													_tmp25_ = x1;
													_tmp26_ = y1;
													_tmp22_[(((_tmp23_ * size) + _tmp24_) * _tmp22__length2) + ((_tmp25_ * size) + _tmp26_)] = _tmp17_;
													_tmp27_ = _tmp22_[(((_tmp23_ * size) + _tmp24_) * _tmp22__length2) + ((_tmp25_ * size) + _tmp26_)];
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	non_pivot_cols = NULL;
	ipiv = 0;
	{
		gint jpiv = 0;
		jpiv = 0;
		{
			gboolean _tmp28_ = FALSE;
			_tmp28_ = TRUE;
			while (TRUE) {
				gint _tmp30_;
				gboolean is_pivot_col = FALSE;
				gboolean _tmp81_;
				if (!_tmp28_) {
					gint _tmp29_;
					_tmp29_ = jpiv;
					jpiv = _tmp29_ + 1;
				}
				_tmp28_ = FALSE;
				_tmp30_ = jpiv;
				if (!(_tmp30_ < (size * size))) {
					break;
				}
				is_pivot_col = FALSE;
				{
					gint i = 0;
					gint _tmp31_;
					_tmp31_ = ipiv;
					i = _tmp31_;
					{
						gboolean _tmp32_ = FALSE;
						_tmp32_ = TRUE;
						while (TRUE) {
							gint _tmp34_;
							gint* _tmp35_;
							gint _tmp35__length1;
							gint _tmp35__length2;
							gint _tmp36_;
							gint _tmp37_;
							gint _tmp38_;
							if (!_tmp32_) {
								gint _tmp33_;
								_tmp33_ = i;
								i = _tmp33_ + 1;
							}
							_tmp32_ = FALSE;
							_tmp34_ = i;
							if (!(_tmp34_ < (size * size))) {
								break;
							}
							_tmp35_ = adj_matrix;
							_tmp35__length1 = adj_matrix_length1;
							_tmp35__length2 = adj_matrix_length2;
							_tmp36_ = i;
							_tmp37_ = jpiv;
							_tmp38_ = _tmp35_[(_tmp36_ * _tmp35__length2) + _tmp37_];
							if (_tmp38_ != 0) {
								gint _tmp39_;
								gint _tmp40_;
								gint _tmp80_;
								_tmp39_ = i;
								_tmp40_ = ipiv;
								if (_tmp39_ != _tmp40_) {
									{
										gint z = 0;
										z = 0;
										{
											gboolean _tmp41_ = FALSE;
											_tmp41_ = TRUE;
											while (TRUE) {
												gint _tmp43_;
												gint t = 0;
												gint* _tmp44_;
												gint _tmp44__length1;
												gint _tmp44__length2;
												gint _tmp45_;
												gint _tmp46_;
												gint _tmp47_;
												gint* _tmp48_;
												gint _tmp48__length1;
												gint _tmp48__length2;
												gint _tmp49_;
												gint _tmp50_;
												gint* _tmp51_;
												gint _tmp51__length1;
												gint _tmp51__length2;
												gint _tmp52_;
												gint _tmp53_;
												gint _tmp54_;
												gint _tmp55_;
												gint* _tmp56_;
												gint _tmp56__length1;
												gint _tmp56__length2;
												gint _tmp57_;
												gint _tmp58_;
												gint _tmp59_;
												gint _tmp60_;
												if (!_tmp41_) {
													gint _tmp42_;
													_tmp42_ = z;
													z = _tmp42_ + 1;
												}
												_tmp41_ = FALSE;
												_tmp43_ = z;
												if (!(_tmp43_ < (size * size))) {
													break;
												}
												_tmp44_ = adj_matrix;
												_tmp44__length1 = adj_matrix_length1;
												_tmp44__length2 = adj_matrix_length2;
												_tmp45_ = i;
												_tmp46_ = z;
												_tmp47_ = _tmp44_[(_tmp45_ * _tmp44__length2) + _tmp46_];
												t = _tmp47_;
												_tmp48_ = adj_matrix;
												_tmp48__length1 = adj_matrix_length1;
												_tmp48__length2 = adj_matrix_length2;
												_tmp49_ = i;
												_tmp50_ = z;
												_tmp51_ = adj_matrix;
												_tmp51__length1 = adj_matrix_length1;
												_tmp51__length2 = adj_matrix_length2;
												_tmp52_ = ipiv;
												_tmp53_ = z;
												_tmp54_ = _tmp51_[(_tmp52_ * _tmp51__length2) + _tmp53_];
												_tmp48_[(_tmp49_ * _tmp48__length2) + _tmp50_] = _tmp54_;
												_tmp55_ = _tmp48_[(_tmp49_ * _tmp48__length2) + _tmp50_];
												_tmp56_ = adj_matrix;
												_tmp56__length1 = adj_matrix_length1;
												_tmp56__length2 = adj_matrix_length2;
												_tmp57_ = ipiv;
												_tmp58_ = z;
												_tmp59_ = t;
												_tmp56_[(_tmp57_ * _tmp56__length2) + _tmp58_] = _tmp59_;
												_tmp60_ = _tmp56_[(_tmp57_ * _tmp56__length2) + _tmp58_];
											}
										}
									}
								}
								{
									gint j = 0;
									gint _tmp61_;
									_tmp61_ = ipiv;
									j = _tmp61_ + 1;
									{
										gboolean _tmp62_ = FALSE;
										_tmp62_ = TRUE;
										while (TRUE) {
											gint _tmp64_;
											gint* _tmp65_;
											gint _tmp65__length1;
											gint _tmp65__length2;
											gint _tmp66_;
											gint _tmp67_;
											gint _tmp68_;
											if (!_tmp62_) {
												gint _tmp63_;
												_tmp63_ = j;
												j = _tmp63_ + 1;
											}
											_tmp62_ = FALSE;
											_tmp64_ = j;
											if (!(_tmp64_ < (size * size))) {
												break;
											}
											_tmp65_ = adj_matrix;
											_tmp65__length1 = adj_matrix_length1;
											_tmp65__length2 = adj_matrix_length2;
											_tmp66_ = j;
											_tmp67_ = jpiv;
											_tmp68_ = _tmp65_[(_tmp66_ * _tmp65__length2) + _tmp67_];
											if (_tmp68_ != 0) {
												{
													gint k = 0;
													k = 0;
													{
														gboolean _tmp69_ = FALSE;
														_tmp69_ = TRUE;
														while (TRUE) {
															gint _tmp71_;
															gint* _tmp72_;
															gint _tmp72__length1;
															gint _tmp72__length2;
															gint _tmp73_;
															gint _tmp74_;
															gint* _tmp75_;
															gint _tmp75__length1;
															gint _tmp75__length2;
															gint _tmp76_;
															gint _tmp77_;
															gint _tmp78_;
															gint _tmp79_;
															if (!_tmp69_) {
																gint _tmp70_;
																_tmp70_ = k;
																k = _tmp70_ + 1;
															}
															_tmp69_ = FALSE;
															_tmp71_ = k;
															if (!(_tmp71_ < (size * size))) {
																break;
															}
															_tmp72_ = adj_matrix;
															_tmp72__length1 = adj_matrix_length1;
															_tmp72__length2 = adj_matrix_length2;
															_tmp73_ = j;
															_tmp74_ = k;
															_tmp75_ = adj_matrix;
															_tmp75__length1 = adj_matrix_length1;
															_tmp75__length2 = adj_matrix_length2;
															_tmp76_ = ipiv;
															_tmp77_ = k;
															_tmp78_ = _tmp75_[(_tmp76_ * _tmp75__length2) + _tmp77_];
															_tmp72_[(_tmp73_ * _tmp72__length2) + _tmp74_] ^= _tmp78_;
															_tmp79_ = _tmp72_[(_tmp73_ * _tmp72__length2) + _tmp74_];
														}
													}
												}
											}
										}
									}
								}
								is_pivot_col = TRUE;
								_tmp80_ = ipiv;
								ipiv = _tmp80_ + 1;
								break;
							}
						}
					}
				}
				_tmp81_ = is_pivot_col;
				if (!_tmp81_) {
					gint _tmp82_;
					_tmp82_ = jpiv;
					non_pivot_cols = g_list_append (non_pivot_cols, (gpointer) ((gintptr) _tmp82_));
				}
			}
		}
	}
	_tmp83_ = non_pivot_cols;
	_tmp84_ = g_new0 (gint, g_list_length (_tmp83_) * (size * size));
	basis_for_ns = _tmp84_;
	basis_for_ns_length1 = g_list_length (_tmp83_);
	basis_for_ns_length2 = size * size;
	n = 0;
	_tmp85_ = non_pivot_cols;
	{
		GList* col_collection = NULL;
		GList* col_it = NULL;
		col_collection = _tmp85_;
		for (col_it = col_collection; col_it != NULL; col_it = col_it->next) {
			gint col = 0;
			col = (gint) ((gintptr) col_it->data);
			{
				gint* _tmp93_;
				gint _tmp93__length1;
				gint _tmp93__length2;
				gint _tmp94_;
				gint _tmp95_;
				gint _tmp96_;
				gint _tmp124_;
				{
					gint j = 0;
					j = 0;
					{
						gboolean _tmp86_ = FALSE;
						_tmp86_ = TRUE;
						while (TRUE) {
							gint _tmp88_;
							gint* _tmp89_;
							gint _tmp89__length1;
							gint _tmp89__length2;
							gint _tmp90_;
							gint _tmp91_;
							gint _tmp92_;
							if (!_tmp86_) {
								gint _tmp87_;
								_tmp87_ = j;
								j = _tmp87_ + 1;
							}
							_tmp86_ = FALSE;
							_tmp88_ = j;
							if (!(_tmp88_ < (size * size))) {
								break;
							}
							_tmp89_ = basis_for_ns;
							_tmp89__length1 = basis_for_ns_length1;
							_tmp89__length2 = basis_for_ns_length2;
							_tmp90_ = n;
							_tmp91_ = j;
							_tmp89_[(_tmp90_ * _tmp89__length2) + _tmp91_] = 0;
							_tmp92_ = _tmp89_[(_tmp90_ * _tmp89__length2) + _tmp91_];
						}
					}
				}
				_tmp93_ = basis_for_ns;
				_tmp93__length1 = basis_for_ns_length1;
				_tmp93__length2 = basis_for_ns_length2;
				_tmp94_ = n;
				_tmp95_ = col;
				_tmp93_[(_tmp94_ * _tmp93__length2) + _tmp95_] = 1;
				_tmp96_ = _tmp93_[(_tmp94_ * _tmp93__length2) + _tmp95_];
				{
					gint i = 0;
					i = (size * size) - 1;
					{
						gboolean _tmp97_ = FALSE;
						_tmp97_ = TRUE;
						while (TRUE) {
							gint _tmp99_;
							gint jpiv = 0;
							gint _tmp107_;
							if (!_tmp97_) {
								gint _tmp98_;
								_tmp98_ = i;
								i = _tmp98_ - 1;
							}
							_tmp97_ = FALSE;
							_tmp99_ = i;
							if (!(_tmp99_ >= 0)) {
								break;
							}
							jpiv = 0;
							{
								gboolean _tmp100_ = FALSE;
								_tmp100_ = TRUE;
								while (TRUE) {
									gint _tmp102_;
									gint* _tmp103_;
									gint _tmp103__length1;
									gint _tmp103__length2;
									gint _tmp104_;
									gint _tmp105_;
									gint _tmp106_;
									if (!_tmp100_) {
										gint _tmp101_;
										_tmp101_ = jpiv;
										jpiv = _tmp101_ + 1;
									}
									_tmp100_ = FALSE;
									_tmp102_ = jpiv;
									if (!(_tmp102_ < (size * size))) {
										break;
									}
									_tmp103_ = adj_matrix;
									_tmp103__length1 = adj_matrix_length1;
									_tmp103__length2 = adj_matrix_length2;
									_tmp104_ = i;
									_tmp105_ = jpiv;
									_tmp106_ = _tmp103_[(_tmp104_ * _tmp103__length2) + _tmp105_];
									if (_tmp106_ != 0) {
										break;
									}
								}
							}
							_tmp107_ = jpiv;
							if (_tmp107_ == (size * size)) {
								continue;
							}
							{
								gint j = 0;
								gint _tmp108_;
								_tmp108_ = jpiv;
								j = _tmp108_ + 1;
								{
									gboolean _tmp109_ = FALSE;
									_tmp109_ = TRUE;
									while (TRUE) {
										gint _tmp111_;
										gint* _tmp112_;
										gint _tmp112__length1;
										gint _tmp112__length2;
										gint _tmp113_;
										gint _tmp114_;
										gint* _tmp115_;
										gint _tmp115__length1;
										gint _tmp115__length2;
										gint _tmp116_;
										gint _tmp117_;
										gint _tmp118_;
										gint* _tmp119_;
										gint _tmp119__length1;
										gint _tmp119__length2;
										gint _tmp120_;
										gint _tmp121_;
										gint _tmp122_;
										gint _tmp123_;
										if (!_tmp109_) {
											gint _tmp110_;
											_tmp110_ = j;
											j = _tmp110_ + 1;
										}
										_tmp109_ = FALSE;
										_tmp111_ = j;
										if (!(_tmp111_ < (size * size))) {
											break;
										}
										_tmp112_ = basis_for_ns;
										_tmp112__length1 = basis_for_ns_length1;
										_tmp112__length2 = basis_for_ns_length2;
										_tmp113_ = n;
										_tmp114_ = jpiv;
										_tmp115_ = adj_matrix;
										_tmp115__length1 = adj_matrix_length1;
										_tmp115__length2 = adj_matrix_length2;
										_tmp116_ = i;
										_tmp117_ = j;
										_tmp118_ = _tmp115_[(_tmp116_ * _tmp115__length2) + _tmp117_];
										_tmp119_ = basis_for_ns;
										_tmp119__length1 = basis_for_ns_length1;
										_tmp119__length2 = basis_for_ns_length2;
										_tmp120_ = n;
										_tmp121_ = j;
										_tmp122_ = _tmp119_[(_tmp120_ * _tmp119__length2) + _tmp121_];
										_tmp112_[(_tmp113_ * _tmp112__length2) + _tmp114_] ^= _tmp118_ * _tmp122_;
										_tmp123_ = _tmp112_[(_tmp113_ * _tmp112__length2) + _tmp114_];
									}
								}
							}
						}
					}
				}
				_tmp124_ = n;
				n = _tmp124_ + 1;
			}
		}
	}
	_tmp125_ = non_pivot_cols;
	_tmp126_ = g_new0 (gint, 1 << g_list_length (_tmp125_));
	self->priv->region_size = (g_free (self->priv->region_size), NULL);
	self->priv->region_size = _tmp126_;
	self->priv->region_size_length1 = 1 << g_list_length (_tmp125_);
	self->priv->_region_size_size_ = self->priv->region_size_length1;
	{
		gint j = 0;
		j = 0;
		{
			gboolean _tmp127_ = FALSE;
			_tmp127_ = TRUE;
			while (TRUE) {
				gint _tmp129_;
				gint* _tmp130_;
				gint _tmp130__length1;
				gint* _tmp131_;
				gint _tmp131__length1;
				gint _tmp132_;
				gint _tmp133_;
				if (!_tmp127_) {
					gint _tmp128_;
					_tmp128_ = j;
					j = _tmp128_ + 1;
				}
				_tmp127_ = FALSE;
				_tmp129_ = j;
				_tmp130_ = self->priv->region_size;
				_tmp130__length1 = self->priv->region_size_length1;
				if (!(_tmp129_ < _tmp130__length1)) {
					break;
				}
				_tmp131_ = self->priv->region_size;
				_tmp131__length1 = self->priv->region_size_length1;
				_tmp132_ = j;
				_tmp131_[_tmp132_] = 0;
				_tmp133_ = _tmp131_[_tmp132_];
			}
		}
	}
	_tmp134_ = g_new0 (gint, size * size);
	self->priv->region_of = (g_free (self->priv->region_of), NULL);
	self->priv->region_of = _tmp134_;
	self->priv->region_of_length1 = size * size;
	self->priv->_region_of_size_ = self->priv->region_of_length1;
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp135_ = FALSE;
			_tmp135_ = TRUE;
			while (TRUE) {
				gint _tmp137_;
				gint* _tmp138_;
				gint _tmp138__length1;
				gint _tmp139_;
				gint _tmp140_;
				gint* _tmp153_;
				gint _tmp153__length1;
				gint* _tmp154_;
				gint _tmp154__length1;
				gint _tmp155_;
				gint _tmp156_;
				gint _tmp157_;
				if (!_tmp135_) {
					gint _tmp136_;
					_tmp136_ = i;
					i = _tmp136_ + 1;
				}
				_tmp135_ = FALSE;
				_tmp137_ = i;
				if (!(_tmp137_ < (size * size))) {
					break;
				}
				_tmp138_ = self->priv->region_of;
				_tmp138__length1 = self->priv->region_of_length1;
				_tmp139_ = i;
				_tmp138_[_tmp139_] = 0;
				_tmp140_ = _tmp138_[_tmp139_];
				{
					gint j = 0;
					j = 0;
					{
						gboolean _tmp141_ = FALSE;
						_tmp141_ = TRUE;
						while (TRUE) {
							gint _tmp143_;
							GList* _tmp144_;
							gint* _tmp145_;
							gint _tmp145__length1;
							gint _tmp145__length2;
							gint _tmp146_;
							gint _tmp147_;
							gint _tmp148_;
							if (!_tmp141_) {
								gint _tmp142_;
								_tmp142_ = j;
								j = _tmp142_ + 1;
							}
							_tmp141_ = FALSE;
							_tmp143_ = j;
							_tmp144_ = non_pivot_cols;
							if (!(((guint) _tmp143_) < g_list_length (_tmp144_))) {
								break;
							}
							_tmp145_ = basis_for_ns;
							_tmp145__length1 = basis_for_ns_length1;
							_tmp145__length2 = basis_for_ns_length2;
							_tmp146_ = j;
							_tmp147_ = i;
							_tmp148_ = _tmp145_[(_tmp146_ * _tmp145__length2) + _tmp147_];
							if (_tmp148_ != 0) {
								gint* _tmp149_;
								gint _tmp149__length1;
								gint _tmp150_;
								gint _tmp151_;
								gint _tmp152_;
								_tmp149_ = self->priv->region_of;
								_tmp149__length1 = self->priv->region_of_length1;
								_tmp150_ = i;
								_tmp151_ = j;
								_tmp149_[_tmp150_] += 1 << _tmp151_;
								_tmp152_ = _tmp149_[_tmp150_];
							}
						}
					}
				}
				_tmp153_ = self->priv->region_size;
				_tmp153__length1 = self->priv->region_size_length1;
				_tmp154_ = self->priv->region_of;
				_tmp154__length1 = self->priv->region_of_length1;
				_tmp155_ = i;
				_tmp156_ = _tmp154_[_tmp155_];
				_tmp157_ = _tmp153_[_tmp156_];
				_tmp153_[_tmp156_] = _tmp157_ + 1;
			}
		}
	}
	_tmp158_ = self->priv->region_size;
	_tmp158__length1 = self->priv->region_size_length1;
	_tmp159_ = _tmp158_[0];
	self->priv->max_solution_length = _tmp159_;
	{
		gint j = 0;
		j = 1;
		{
			gboolean _tmp160_ = FALSE;
			_tmp160_ = TRUE;
			while (TRUE) {
				gint _tmp162_;
				gint* _tmp163_;
				gint _tmp163__length1;
				gint _tmp164_;
				gint* _tmp165_;
				gint _tmp165__length1;
				gint _tmp166_;
				gint _tmp167_;
				if (!_tmp160_) {
					gint _tmp161_;
					_tmp161_ = j;
					j = _tmp161_ + 1;
				}
				_tmp160_ = FALSE;
				_tmp162_ = j;
				_tmp163_ = self->priv->region_size;
				_tmp163__length1 = self->priv->region_size_length1;
				if (!(_tmp162_ < _tmp163__length1)) {
					break;
				}
				_tmp164_ = self->priv->max_solution_length;
				_tmp165_ = self->priv->region_size;
				_tmp165__length1 = self->priv->region_size_length1;
				_tmp166_ = j;
				_tmp167_ = _tmp165_[_tmp166_];
				self->priv->max_solution_length = _tmp164_ + ((gint) floor ((gdouble) (_tmp167_ / 2)));
			}
		}
	}
	basis_for_ns = (g_free (basis_for_ns), NULL);
	(non_pivot_cols == NULL) ? NULL : (non_pivot_cols = (g_list_free (non_pivot_cols), NULL));
	adj_matrix = (g_free (adj_matrix), NULL);
	return self;
}


PuzzleGenerator*
puzzle_generator_new (gint size)
{
	return puzzle_generator_construct (TYPE_PUZZLE_GENERATOR, size);
}


gboolean*
puzzle_generator_minimal_solution (PuzzleGenerator* self,
                                   gint solution_length,
                                   int* result_length1,
                                   int* result_length2)
{
	gboolean* result = NULL;
	gboolean* sol = NULL;
	gint _tmp0_;
	gint _tmp1_;
	gboolean* _tmp2_;
	gint sol_length1;
	gint sol_length2;
	gint* presses_in_region = NULL;
	gint* _tmp15_;
	gint _tmp15__length1;
	gint* _tmp16_;
	gint presses_in_region_length1;
	gint _presses_in_region_size_;
	gint sym = 0;
	gint presses_left = 0;
	gint _tmp24_;
	gboolean* _tmp83_;
	gint _tmp83__length1;
	gint _tmp83__length2;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->size;
	_tmp1_ = self->priv->size;
	_tmp2_ = g_new0 (gboolean, _tmp0_ * _tmp1_);
	sol = _tmp2_;
	sol_length1 = _tmp0_;
	sol_length2 = _tmp1_;
	{
		gint x = 0;
		x = 0;
		{
			gboolean _tmp3_ = FALSE;
			_tmp3_ = TRUE;
			while (TRUE) {
				gint _tmp5_;
				gint _tmp6_;
				if (!_tmp3_) {
					gint _tmp4_;
					_tmp4_ = x;
					x = _tmp4_ + 1;
				}
				_tmp3_ = FALSE;
				_tmp5_ = x;
				_tmp6_ = self->priv->size;
				if (!(_tmp5_ < _tmp6_)) {
					break;
				}
				{
					gint y = 0;
					y = 0;
					{
						gboolean _tmp7_ = FALSE;
						_tmp7_ = TRUE;
						while (TRUE) {
							gint _tmp9_;
							gint _tmp10_;
							gboolean* _tmp11_;
							gint _tmp11__length1;
							gint _tmp11__length2;
							gint _tmp12_;
							gint _tmp13_;
							gboolean _tmp14_;
							if (!_tmp7_) {
								gint _tmp8_;
								_tmp8_ = y;
								y = _tmp8_ + 1;
							}
							_tmp7_ = FALSE;
							_tmp9_ = y;
							_tmp10_ = self->priv->size;
							if (!(_tmp9_ < _tmp10_)) {
								break;
							}
							_tmp11_ = sol;
							_tmp11__length1 = sol_length1;
							_tmp11__length2 = sol_length2;
							_tmp12_ = x;
							_tmp13_ = y;
							_tmp11_[(_tmp12_ * _tmp11__length2) + _tmp13_] = FALSE;
							_tmp14_ = _tmp11_[(_tmp12_ * _tmp11__length2) + _tmp13_];
						}
					}
				}
			}
		}
	}
	_tmp15_ = self->priv->region_size;
	_tmp15__length1 = self->priv->region_size_length1;
	_tmp16_ = g_new0 (gint, _tmp15__length1);
	presses_in_region = _tmp16_;
	presses_in_region_length1 = _tmp15__length1;
	_presses_in_region_size_ = presses_in_region_length1;
	{
		gint i = 0;
		i = 0;
		{
			gboolean _tmp17_ = FALSE;
			_tmp17_ = TRUE;
			while (TRUE) {
				gint _tmp19_;
				gint* _tmp20_;
				gint _tmp20__length1;
				gint* _tmp21_;
				gint _tmp21__length1;
				gint _tmp22_;
				gint _tmp23_;
				if (!_tmp17_) {
					gint _tmp18_;
					_tmp18_ = i;
					i = _tmp18_ + 1;
				}
				_tmp17_ = FALSE;
				_tmp19_ = i;
				_tmp20_ = self->priv->region_size;
				_tmp20__length1 = self->priv->region_size_length1;
				if (!(_tmp19_ < _tmp20__length1)) {
					break;
				}
				_tmp21_ = presses_in_region;
				_tmp21__length1 = presses_in_region_length1;
				_tmp22_ = i;
				_tmp21_[_tmp22_] = 0;
				_tmp23_ = _tmp21_[_tmp22_];
			}
		}
	}
	sym = (gint) floor (3 * g_random_double ());
	_tmp24_ = self->priv->max_solution_length;
	presses_left = MIN (solution_length, _tmp24_);
	while (TRUE) {
		gint _tmp25_;
		gint x[2] = {0};
		gint y[2] = {0};
		gint _tmp26_;
		gint _tmp27_;
		gint _tmp28_;
		gint _tmp29_;
		gint _tmp30_;
		_tmp25_ = presses_left;
		if (!(_tmp25_ > 0)) {
			break;
		}
		_tmp26_ = self->priv->size;
		x[0] = (gint) round ((_tmp26_ - 1) * g_random_double ());
		_tmp27_ = x[0];
		_tmp28_ = self->priv->size;
		y[0] = (gint) round ((_tmp28_ - 1) * g_random_double ());
		_tmp29_ = y[0];
		_tmp30_ = sym;
		if (_tmp30_ == 0) {
			gint _tmp31_;
			gint _tmp32_;
			gint _tmp33_;
			gint _tmp34_;
			gint _tmp35_;
			_tmp31_ = self->priv->size;
			_tmp32_ = x[0];
			x[1] = (_tmp31_ - 1) - _tmp32_;
			_tmp33_ = x[1];
			_tmp34_ = y[0];
			y[1] = _tmp34_;
			_tmp35_ = y[1];
		} else {
			gint _tmp36_;
			_tmp36_ = sym;
			if (_tmp36_ == 1) {
				gint _tmp37_;
				gint _tmp38_;
				gint _tmp39_;
				gint _tmp40_;
				gint _tmp41_;
				gint _tmp42_;
				_tmp37_ = self->priv->size;
				_tmp38_ = x[0];
				x[1] = (_tmp37_ - 1) - _tmp38_;
				_tmp39_ = x[1];
				_tmp40_ = self->priv->size;
				_tmp41_ = y[0];
				y[1] = (_tmp40_ - 1) - _tmp41_;
				_tmp42_ = y[1];
			} else {
				gint _tmp43_;
				gint _tmp44_;
				gint _tmp45_;
				gint _tmp46_;
				gint _tmp47_;
				_tmp43_ = x[0];
				x[1] = _tmp43_;
				_tmp44_ = x[1];
				_tmp45_ = self->priv->size;
				_tmp46_ = y[0];
				y[1] = (_tmp45_ - 1) - _tmp46_;
				_tmp47_ = y[1];
			}
		}
		{
			gint k = 0;
			k = 0;
			{
				gboolean _tmp48_ = FALSE;
				_tmp48_ = TRUE;
				while (TRUE) {
					gint _tmp50_;
					gint r = 0;
					gint* _tmp51_;
					gint _tmp51__length1;
					gint _tmp52_;
					gint _tmp53_;
					gint _tmp54_;
					gint _tmp55_;
					gint _tmp56_;
					gint _tmp57_;
					gboolean _tmp58_ = FALSE;
					gint _tmp59_;
					gint _tmp82_;
					if (!_tmp48_) {
						gint _tmp49_;
						_tmp49_ = k;
						k = _tmp49_ + 1;
					}
					_tmp48_ = FALSE;
					_tmp50_ = k;
					if (!(_tmp50_ < 2)) {
						break;
					}
					_tmp51_ = self->priv->region_of;
					_tmp51__length1 = self->priv->region_of_length1;
					_tmp52_ = k;
					_tmp53_ = x[_tmp52_];
					_tmp54_ = self->priv->size;
					_tmp55_ = k;
					_tmp56_ = y[_tmp55_];
					_tmp57_ = _tmp51_[(_tmp53_ * _tmp54_) + _tmp56_];
					r = _tmp57_;
					_tmp59_ = r;
					if (_tmp59_ == 0) {
						_tmp58_ = TRUE;
					} else {
						gint* _tmp60_;
						gint _tmp60__length1;
						gint _tmp61_;
						gint _tmp62_;
						gint* _tmp63_;
						gint _tmp63__length1;
						gint _tmp64_;
						gint _tmp65_;
						_tmp60_ = presses_in_region;
						_tmp60__length1 = presses_in_region_length1;
						_tmp61_ = r;
						_tmp62_ = _tmp60_[_tmp61_];
						_tmp63_ = self->priv->region_size;
						_tmp63__length1 = self->priv->region_size_length1;
						_tmp64_ = r;
						_tmp65_ = _tmp63_[_tmp64_];
						_tmp58_ = (2 * (_tmp62_ + 1)) <= _tmp65_;
					}
					if (_tmp58_) {
						gboolean* _tmp66_;
						gint _tmp66__length1;
						gint _tmp66__length2;
						gint _tmp67_;
						gint _tmp68_;
						gint _tmp69_;
						gint _tmp70_;
						gboolean _tmp71_;
						gboolean* _tmp72_;
						gint _tmp72__length1;
						gint _tmp72__length2;
						gint _tmp73_;
						gint _tmp74_;
						gint _tmp75_;
						gint _tmp76_;
						gboolean _tmp77_;
						gint* _tmp78_;
						gint _tmp78__length1;
						gint _tmp79_;
						gint _tmp80_;
						gint _tmp81_;
						_tmp66_ = sol;
						_tmp66__length1 = sol_length1;
						_tmp66__length2 = sol_length2;
						_tmp67_ = k;
						_tmp68_ = x[_tmp67_];
						_tmp69_ = k;
						_tmp70_ = y[_tmp69_];
						_tmp71_ = _tmp66_[(_tmp68_ * _tmp66__length2) + _tmp70_];
						if (_tmp71_) {
							continue;
						}
						_tmp72_ = sol;
						_tmp72__length1 = sol_length1;
						_tmp72__length2 = sol_length2;
						_tmp73_ = k;
						_tmp74_ = x[_tmp73_];
						_tmp75_ = k;
						_tmp76_ = y[_tmp75_];
						_tmp72_[(_tmp74_ * _tmp72__length2) + _tmp76_] = TRUE;
						_tmp77_ = _tmp72_[(_tmp74_ * _tmp72__length2) + _tmp76_];
						_tmp78_ = presses_in_region;
						_tmp78__length1 = presses_in_region_length1;
						_tmp79_ = r;
						_tmp80_ = _tmp78_[_tmp79_];
						_tmp78_[_tmp79_] = _tmp80_ + 1;
						_tmp81_ = presses_left;
						presses_left = _tmp81_ - 1;
					}
					_tmp82_ = presses_left;
					if (_tmp82_ == 0) {
						break;
					}
				}
			}
		}
	}
	_tmp83_ = sol;
	_tmp83__length1 = sol_length1;
	_tmp83__length2 = sol_length2;
	if (result_length1) {
		*result_length1 = _tmp83__length1;
	}
	if (result_length2) {
		*result_length2 = _tmp83__length2;
	}
	result = _tmp83_;
	presses_in_region = (g_free (presses_in_region), NULL);
	return result;
}


static void
puzzle_generator_class_init (PuzzleGeneratorClass * klass)
{
	puzzle_generator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &PuzzleGenerator_private_offset);
	G_OBJECT_CLASS (klass)->finalize = puzzle_generator_finalize;
}


static void
puzzle_generator_instance_init (PuzzleGenerator * self)
{
	self->priv = puzzle_generator_get_instance_private (self);
}


static void
puzzle_generator_finalize (GObject * obj)
{
	PuzzleGenerator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_PUZZLE_GENERATOR, PuzzleGenerator);
	self->priv->region_of = (g_free (self->priv->region_of), NULL);
	self->priv->region_size = (g_free (self->priv->region_size), NULL);
	G_OBJECT_CLASS (puzzle_generator_parent_class)->finalize (obj);
}


GType
puzzle_generator_get_type (void)
{
	static volatile gsize puzzle_generator_type_id__volatile = 0;
	if (g_once_init_enter (&puzzle_generator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (PuzzleGeneratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) puzzle_generator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PuzzleGenerator), 0, (GInstanceInitFunc) puzzle_generator_instance_init, NULL };
		GType puzzle_generator_type_id;
		puzzle_generator_type_id = g_type_register_static (G_TYPE_OBJECT, "PuzzleGenerator", &g_define_type_info, 0);
		PuzzleGenerator_private_offset = g_type_add_instance_private (puzzle_generator_type_id, sizeof (PuzzleGeneratorPrivate));
		g_once_init_leave (&puzzle_generator_type_id__volatile, puzzle_generator_type_id);
	}
	return puzzle_generator_type_id__volatile;
}



